<?php

namespace Acms\Services\Logger\Handler;

use Monolog\Handler\AbstractProcessingHandler;
use Acms\Services\Logger\Level;
use Acms\Services\Facades\Mailer;
use Field;
use DB;
use SQL;
use Storage;
use Template;
use Tpl;
use ACMS_Corrector;
use ACMS_RAM;


class EmailHandler extends AbstractProcessingHandler
{
    /**
     * ログをメールで送信
     * @param array $record
     * @return void
     */
    protected function write(array $record): void
    {
        try {
            $field = new Field;
            $field->addField('channel', $record['channel']);
            $field->addField('level', Level::getLevelNameJa(intval($record['level'])));
            $field->addField('message', $record['message']);
            $field->addField('formatted', $record['formatted']);
            $field->addField('datetime', date('Y-m-d H:i:s', $record['datetime']->format('U')));
            if (defined('RBID')) {
                $field->addField('rootBlogName', trim(ACMS_RAM::blogName(RBID)));
            } else if (defined('DOMAIN')) {
                $field->addField('rootBlogName', DOMAIN);
            }
            $field->addField('url', REQUEST_URL);
            $field->addField('remoteAddr', REMOTE_ADDR);
            $field->addField('auditLogUrl', BASE_URL . 'bid/1/admin/audit_log/');

            $subjectTpl = findTemplate('mail/logger/subject.txt');
            $bodyTpl = findTemplate('mail/logger/body.txt');
            if (empty($subjectTpl) || empty($bodyTpl)) {
                throw new \RuntimeException('Not found mail template for logger.');
            }
            $subject = $this->buildMailTxt($subjectTpl, $field);
            $body = $this->buildMailTxt($bodyTpl, $field);
            $to = $this->getToAddress();
            $from = env('ALERT_EMAIL_FROM', 'info@example.com');
            if (empty($to) || empty($from)) {
                throw new \RuntimeException('No to address or from address specified.');
            }
            $mailer = Mailer::init();
            $mailer = $mailer->setFrom($from)
                ->setTo($to)
                ->setBcc(env('ALERT_EMAIL_BCC', ''))
                ->setSubject($subject)
                ->setBody($body);
            $mailer->send();

        } catch (\Exception $e) {
        }
    }

    protected function buildMailTxt($tplPath, $field)
    {
        $tplTxt = Storage::get($tplPath);
        $tpl = new Template($tplTxt, new ACMS_Corrector());
        $vars = Tpl::buildField($field, $tpl);
        $tpl->add(null, $vars);

        return buildIF($tpl->get());
    }

    /**
     * エラー通知する宛先のアドレスを取得
     * @return bool|string
     */
    protected function getToAddress()
    {
        if ($email = env('ALERT_EMAIL_TO', false)) {
            return $email;
        }
        $sql = SQL::newSelect('user');
        $sql->setSelect('user_mail');
        $sql->addWhereOpr('user_status', 'open');
        $sql->addWhereOpr('user_login_expire', date('Y-m-d', REQUEST_TIME), '>=');
        $sql->addWhereOpr('user_auth', 'administrator');
        $sql->setOrder('user_id', 'ASC');
        $sql->setLimit(1);

        return DB::query($sql->get(dsn()), 'one');
    }
}
